@@ -25,14 +25,20 @@
  * - wm communication
  */
 
-#define DEFAULT_SPACING 6
-#define DEFAULT_HPADDING 6
+#define DEFAULT_SPACING 8
+#define DEFAULT_HPADDING 8
 #define DEFAULT_VPADDING 6
 
 struct _GdHeaderBarPrivate
 {
   gchar *title;
-  GtkWidget *label;
+  gchar *subtitle;
+
+  GtkWidget *title_label;
+  GtkWidget *subtitle_label;
+  GtkWidget *labels_box;
+  GtkWidget *labels_sizing_box;
+
   GtkWidget *custom_title;
   GdkWindow *event_window;
   gint spacing;
@@ -52,6 +58,7 @@ struct _Child
 enum {
   PROP_0,
   PROP_TITLE,
+  PROP_SUBTITLE,
   PROP_CUSTOM_TITLE,
   PROP_SPACING,
   PROP_HPADDING,
@@ -80,6 +87,18 @@ boldify_label (GtkWidget *label)
   pango_attr_list_unref (attrs);
 }
 
+static void
+smallify_label (GtkWidget *label)
+{
+  PangoAttrList *attrs;
+  attrs = pango_attr_list_new ();
+  pango_attr_list_insert (attrs, pango_attr_scale_new (PANGO_SCALE_SMALL));
+  gtk_label_set_attributes (GTK_LABEL (label), attrs);
+  pango_attr_list_unref (attrs);
+
+  gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
+}
+
 static void
 get_css_padding_and_border (GtkWidget *widget,
                             GtkBorder *border)
@@ -99,6 +118,35 @@ get_css_padding_and_border (GtkWidget *widget,
   border->left += tmp.left;
 }
 
+static void
+init_sizing_box (GdHeaderBar *bar)
+{
+  GdHeaderBarPrivate *priv = bar->priv;
+  GtkWidget *w;
+
+  /* We use this box to always request size for the two labels (title and subtitle)
+   * as if they were always visible, but then allocate the real label box
+   * with its actual size, to keep it center-aligned in case we have only the title.
+   */
+  priv->labels_sizing_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+
+  w = gtk_label_new (NULL);
+  boldify_label (w);
+  gtk_box_pack_start (GTK_BOX (priv->labels_sizing_box), w, FALSE, FALSE, 0);
+  gtk_label_set_line_wrap (GTK_LABEL (w), FALSE);
+  gtk_label_set_single_line_mode (GTK_LABEL (w), TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (w), PANGO_ELLIPSIZE_END);
+
+  w = gtk_label_new (NULL);
+  smallify_label (w);
+  gtk_box_pack_start (GTK_BOX (priv->labels_sizing_box), w, FALSE, FALSE, 0);
+  gtk_label_set_line_wrap (GTK_LABEL (w), FALSE);
+  gtk_label_set_single_line_mode (GTK_LABEL (w), TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (w), PANGO_ELLIPSIZE_END);
+
+  gtk_widget_show_all (priv->labels_sizing_box);
+}
+
 static void
 gd_header_bar_init (GdHeaderBar *bar)
 {
@@ -111,14 +159,27 @@ gd_header_bar_init (GdHeaderBar *bar)
   gtk_widget_set_has_window (GTK_WIDGET (bar), FALSE);
   gtk_widget_set_redraw_on_allocate (GTK_WIDGET (bar), FALSE);
 
-  priv->label = gtk_label_new ("");
-  boldify_label (priv->label);
-  gtk_widget_set_parent (priv->label, GTK_WIDGET (bar));
-  gtk_widget_set_valign (priv->label, GTK_ALIGN_CENTER);
-  gtk_label_set_line_wrap (GTK_LABEL (priv->label), FALSE);
-  gtk_label_set_single_line_mode (GTK_LABEL (priv->label), TRUE);
-  gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
-  gtk_widget_show (priv->label);
+  priv->labels_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_widget_set_parent (priv->labels_box, GTK_WIDGET (bar));
+  gtk_widget_set_valign (priv->labels_box, GTK_ALIGN_CENTER);
+  gtk_widget_show (priv->labels_box);
+
+  init_sizing_box (bar);
+
+  priv->title_label = gtk_label_new ("");
+  boldify_label (priv->title_label);
+  gtk_box_pack_start (GTK_BOX (priv->labels_box), priv->title_label, FALSE, FALSE, 0);
+  gtk_label_set_line_wrap (GTK_LABEL (priv->title_label), FALSE);
+  gtk_label_set_single_line_mode (GTK_LABEL (priv->title_label), TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (priv->title_label), PANGO_ELLIPSIZE_END);
+  gtk_widget_show (priv->title_label);
+
+  priv->subtitle_label = gtk_label_new ("");
+  smallify_label (priv->subtitle_label);
+  gtk_box_pack_start (GTK_BOX (priv->labels_box), priv->subtitle_label, FALSE, FALSE, 0);
+  gtk_label_set_line_wrap (GTK_LABEL (priv->subtitle_label), FALSE);
+  gtk_label_set_single_line_mode (GTK_LABEL (priv->subtitle_label), TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (priv->subtitle_label), PANGO_ELLIPSIZE_END);
 
   priv->title = NULL;
   priv->custom_title = NULL;
@@ -166,6 +227,13 @@ gd_header_bar_get_size (GtkWidget      *widget,
   minimum = natural = 0;
   nvis_children = 0;
 
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    gtk_widget_get_preferred_width (priv->labels_sizing_box,
+                                    &minimum, &natural);
+  else
+    gtk_widget_get_preferred_height (priv->labels_sizing_box,
+                                     &minimum, &natural);
+
   for (l = priv->children; l; l = l->next)
     {
       Child *child = l->data;
@@ -195,8 +263,6 @@ gd_header_bar_get_size (GtkWidget      *widget,
         }
     }
 
-  /* FIXME label size */
-
   if (nvis_children > 0 && orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       minimum += nvis_children * priv->spacing;
@@ -259,7 +325,10 @@ gd_header_bar_compute_size_for_orientation (GtkWidget *widget,
         }
     }
 
-  /* FIXME label size */
+  gtk_widget_get_preferred_width (priv->labels_sizing_box,
+                                  &child_size, &child_natural);
+  required_size += child_size;
+  required_natural += child_natural;
 
   if (nvis_children > 0)
     {
@@ -361,6 +430,11 @@ gd_header_bar_compute_size_for_opposing_orientation (GtkWidget *widget,
       i += 1;
     }
 
+  gtk_widget_get_preferred_height (priv->labels_sizing_box,
+                                   &child_minimum, &child_natural);
+  computed_minimum = MAX (computed_minimum, child_minimum);
+  computed_natural = MAX (computed_natural, child_natural);
+
   get_css_padding_and_border (widget, &css_borders);
 
   computed_minimum += 2 * priv->vpadding + css_borders.top + css_borders.bottom;
@@ -471,7 +545,7 @@ gd_header_bar_size_allocate (GtkWidget     *widget,
     }
   else
     {
-      gtk_widget_get_preferred_width_for_height (priv->label,
+      gtk_widget_get_preferred_width_for_height (priv->labels_box,
                                                  height,
                                                  &title_minimum_size,
                                                  &title_natural_size);
@@ -483,12 +557,12 @@ gd_header_bar_size_allocate (GtkWidget     *widget,
   side[0] = side[1] = 0;
   for (packing = GTK_PACK_START; packing <= GTK_PACK_END; packing++)
     {
-      child_allocation.y = allocation->y + priv->vpadding;
+      child_allocation.y = allocation->y + priv->vpadding + css_borders.top;
       child_allocation.height = height;
       if (packing == GTK_PACK_START)
-        x = allocation->x + priv->hpadding;
+        x = allocation->x + priv->hpadding + css_borders.left;
       else
-        x = allocation->x + allocation->width - priv->hpadding;
+        x = allocation->x + allocation->width - priv->hpadding - css_borders.right;
 
       if (packing == GTK_PACK_START)
 	{
@@ -542,7 +616,7 @@ gd_header_bar_size_allocate (GtkWidget     *widget,
         }
     }
 
-  child_allocation.y = allocation->y + priv->vpadding;
+  child_allocation.y = allocation->y + priv->vpadding + css_borders.top;
   child_allocation.height = height;
 
   width = MAX(side[0], side[1]);
@@ -568,7 +642,7 @@ gd_header_bar_size_allocate (GtkWidget     *widget,
   if (priv->custom_title)
     gtk_widget_size_allocate (priv->custom_title, &child_allocation);
   else
-    gtk_widget_size_allocate (priv->label, &child_allocation);
+    gtk_widget_size_allocate (priv->labels_box, &child_allocation);
 }
 
 /**
@@ -596,7 +670,7 @@ gd_header_bar_set_title (GdHeaderBar *bar,
   g_free (priv->title);
   priv->title = new_title;
 
-  gtk_label_set_label (GTK_LABEL (priv->label), priv->title);
+  gtk_label_set_label (GTK_LABEL (priv->title_label), priv->title);
   gtk_widget_queue_resize (GTK_WIDGET (bar));
 
   g_object_notify (G_OBJECT (bar), "title");
@@ -620,6 +694,56 @@ gd_header_bar_get_title (GdHeaderBar *bar)
   return bar->priv->title;
 }
 
+/**
+ * gd_header_bar_set_subtitle:
+ * @bar: a #GdHeaderBar
+ * @subtitle: (allow-none): a subtitle
+ *
+ * Sets the subtitle of the #GdHeaderBar. The subtitle should give a user
+ * an additional detail to help him identify the current view.
+ *
+ **/
+void
+gd_header_bar_set_subtitle (GdHeaderBar *bar,
+                            const gchar *subtitle)
+{
+  GdHeaderBarPrivate *priv;
+  char *new_subtitle;
+
+  g_return_if_fail (GD_IS_HEADER_BAR (bar));
+
+  priv = bar->priv;
+
+  new_subtitle = g_strdup (subtitle);
+  g_free (priv->subtitle);
+  priv->subtitle = new_subtitle;
+
+  gtk_label_set_label (GTK_LABEL (priv->subtitle_label), priv->subtitle);
+  gtk_widget_set_visible (priv->subtitle_label, (priv->subtitle != NULL));
+
+  gtk_widget_queue_resize (GTK_WIDGET (bar));
+
+  g_object_notify (G_OBJECT (bar), "subtitle");
+}
+
+/**
+ * gd_header_bar_get_subtitle:
+ * @bar: a #GdHeaderBar
+ *
+ * Retrieves the subtitle of the header. See gd_header_bar_set_subtitle().
+ *
+ * Return value: the subtitle of the header, or %NULL if none has
+ *    been set explicitely. The returned string is owned by the widget
+ *    and must not be modified or freed.
+ **/
+const gchar *
+gd_header_bar_get_subtitle (GdHeaderBar *bar)
+{
+  g_return_val_if_fail (GD_IS_HEADER_BAR (bar), NULL);
+
+  return bar->priv->subtitle;
+}
+
 /**
  * gd_header_bar_set_custom_title:
  * @bar: a #GdHeaderBar
@@ -660,7 +784,7 @@ gd_header_bar_set_custom_title (GdHeaderBar *bar,
     {
       priv->custom_title = g_object_ref (title_widget);
 
-      gtk_widget_hide (priv->label);
+      gtk_widget_hide (priv->labels_box);
 
       gtk_widget_set_parent (priv->custom_title, GTK_WIDGET (bar));
       gtk_widget_set_valign (priv->custom_title, GTK_ALIGN_CENTER);
@@ -669,7 +793,7 @@ gd_header_bar_set_custom_title (GdHeaderBar *bar,
     }
   else
     {
-      gtk_widget_show (priv->label);
+      gtk_widget_show (priv->labels_box);
     }
 
   gtk_widget_queue_resize (GTK_WIDGET (bar));
@@ -695,6 +819,20 @@ gd_header_bar_get_custom_title (GdHeaderBar *bar)
   return bar->priv->custom_title;
 }
 
+static void
+gd_header_bar_finalize (GObject *object)
+{
+  GdHeaderBar *bar = GD_HEADER_BAR (object);
+  GdHeaderBarPrivate *priv = bar->priv;
+
+  g_free (priv->title);
+  g_free (priv->subtitle);
+
+  g_list_free (priv->children);
+
+  G_OBJECT_CLASS (gd_header_bar_parent_class)->finalize (object);
+}
+
 static void
 gd_header_bar_get_property (GObject      *object,
                             guint         prop_id,
@@ -710,6 +848,10 @@ gd_header_bar_get_property (GObject      *object,
       g_value_set_string (value, priv->title);
       break;
 
+    case PROP_SUBTITLE:
+      g_value_set_string (value, priv->subtitle);
+      break;
+
     case PROP_CUSTOM_TITLE:
       g_value_set_object (value, priv->custom_title);
       break;
@@ -747,6 +889,10 @@ gd_header_bar_set_property (GObject      *object,
       gd_header_bar_set_title (bar, g_value_get_string (value));
       break;
 
+    case PROP_SUBTITLE:
+      gd_header_bar_set_subtitle (bar, g_value_get_string (value));
+      break;
+
     case PROP_CUSTOM_TITLE:
       gd_header_bar_set_custom_title (bar, g_value_get_object (value));
       break;
@@ -862,7 +1008,7 @@ gd_header_bar_forall (GtkContainer *container,
       if (priv->custom_title)
         (* callback) (priv->custom_title, callback_data);
       else
-        (* callback) (priv->label, callback_data);
+        (* callback) (priv->labels_box, callback_data);
     }
 
   children = g_list_last (priv->children);
@@ -1070,6 +1216,7 @@ gd_header_bar_class_init (GdHeaderBarClass *class)
 
   object_class->get_property = gd_header_bar_get_property;
   object_class->set_property = gd_header_bar_set_property;
+  object_class->finalize = gd_header_bar_finalize;
 
   widget_class->size_allocate = gd_header_bar_size_allocate;
   widget_class->get_preferred_width = gd_header_bar_get_preferred_width;
@@ -1114,6 +1261,14 @@ gd_header_bar_class_init (GdHeaderBarClass *class)
                                                         NULL,
                                                         G_PARAM_READWRITE));
 
+  g_object_class_install_property (object_class,
+                                   PROP_SUBTITLE,
+                                   g_param_spec_string ("subtitle",
+                                                        "Subtitle",
+                                                        "The subtitle to display",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
+
   g_object_class_install_property (object_class,
                                    PROP_CUSTOM_TITLE,
                                    g_param_spec_object ("custom-title",
