@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, 2013 Red Hat, Inc.
+ * Copyright (c) 2011, 2012, 2013, 2014 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -248,8 +248,8 @@ const DocCommon = new Lang.Class({
         this.populateFromCursor(cursor);
 
         this._refreshIconId =
-            Application.settings.connect('changed::view-as',
-                                         Lang.bind(this, this.refreshIcon));
+            Application.application.connect('action-state-changed::view-as',
+                                            Lang.bind(this, this.refreshIcon));
         this._filterId =
             Application.searchCategoryManager.connect('active-changed',
                                                       Lang.bind(this, this.refreshIcon));
@@ -355,6 +355,18 @@ const DocCommon = new Lang.Class({
         }
     },
 
+    canEdit: function() {
+        log('Error: DocCommon implementations must override canEdit');
+    },
+
+    canShare: function() {
+        log('Error: DocCommon implementations must override canShare');
+    },
+
+    canTrash: function() {
+        log('Error: DocCommon implementations must override canTrash');
+    },
+
     createThumbnail: function(callback) {
         log('Error: DocCommon implementations must override createThumbnail');
     },
@@ -448,14 +460,20 @@ const DocCommon = new Lang.Class({
                                     this.thumbnailed = true;
                                     this._setOrigPixbuf(pixbuf);
                                 } catch (e) {
+                                    log('Unable to create pixbuf from ' + thumbFile.get_uri() + ': ' + e.toString());
                                     this._failedThumbnailing = true;
+                                    this._thumbPath = null;
+                                    thumbFile.delete_async(GLib.PRIORITY_DEFAULT, null, null);
                                 }
 
                                 // close the underlying stream immediately
                                 stream.close_async(0, null, null);
                             }));
                 } catch (e) {
+                    log('Unable to read file at ' + thumbFile.get_uri() + ': ' + e.toString());
                     this._failedThumbnailing = true;
+                    this._thumbPath = null;
+                    thumbFile.delete_async(GLib.PRIORITY_DEFAULT, null, null);
                 }
             }));
     },
@@ -543,7 +561,7 @@ const DocCommon = new Lang.Class({
             this._collectionIconWatcher = null;
         }
 
-        Application.settings.disconnect(this._refreshIconId);
+        Application.application.disconnect(this._refreshIconId);
         Application.searchCategoryManager.disconnect(this._filterId);
     },
 
@@ -653,6 +671,14 @@ const LocalDocument = new Lang.Class({
             }));
     },
 
+    canEdit: function() {
+        return false;
+    },
+
+    canShare: function() {
+        return false;
+    },
+
     canTrash: function() {
         return this.collection;
     },
@@ -816,6 +842,14 @@ const GoogleDocument = new Lang.Class({
         this.parent(cursor);
     },
 
+    canEdit: function() {
+        return true;
+    },
+
+    canShare: function() {
+        return true;
+    },
+
     canTrash: function() {
         return false;
     }
@@ -872,6 +906,14 @@ const OwncloudDocument = new Lang.Class({
             }));
     },
 
+    canEdit: function() {
+        return false;
+    },
+
+    canShare: function() {
+        return false;
+    },
+
     canTrash: function() {
         return false;
     }
@@ -964,6 +1006,14 @@ const SkydriveDocument = new Lang.Class({
         this.typeDescription = description;
     },
 
+    canEdit: function() {
+        return false;
+    },
+
+    canShare: function() {
+        return false;
+    },
+
     canTrash: function() {
         return false;
     }
