@@ -34,6 +34,7 @@ const WindowMode = imports.windowMode;
 const Documents = imports.documents;
 
 const EvView = imports.gi.EvinceView;
+const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
@@ -280,6 +281,14 @@ const Embed = new Lang.Class({
             this._onWindowModeChanged(Application.modeController, windowMode, WindowMode.WindowMode.NONE);
     },
 
+    _onActivateResult: function() {
+        let iter = this._view.model.model.get_iter_first()[1];
+        if (iter) {
+            let id = this._view.model.model.get_value(iter, Gd.MainColumns.ID);
+            Application.documentManager.setActiveItem(Application.documentManager.getItemById(id))
+        };
+    },
+
     _onQueryStatusChanged: function() {
         let windowMode = Application.modeController.getWindowMode();
         if (windowMode != WindowMode.WindowMode.OVERVIEW)
@@ -354,6 +363,8 @@ const Embed = new Lang.Class({
             throw(new Error('Not handled'));
             break;
         }
+        this._toolbar.searchbar.connect('activate-result',
+                                        Lang.bind(this, this._onActivateResult));
     },
 
     _onActiveItemChanged: function(manager, doc) {
@@ -415,7 +426,7 @@ const Embed = new Lang.Class({
         dialog.widget.connect('response', Lang.bind(this,
             function(widget, response) {
                 dialog.widget.destroy();
-                if (response == Gtk.ResponseType.CANCEL)
+                if (response == Gtk.ResponseType.CANCEL || response == Gtk.ResponseType.DELETE_EVENT)
                     Application.documentManager.setActiveItem(null);
             }));
     },
@@ -425,6 +436,8 @@ const Embed = new Lang.Class({
             this._preview.setModel(null);
         if (this._edit)
             this._edit.setUri(null);
+        if (this._toolbar)
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
         this._toolbar = new MainToolbar.OverviewToolbar(this._stackOverlay);
@@ -438,6 +451,8 @@ const Embed = new Lang.Class({
     _prepareForPreview: function() {
         if (this._edit)
             this._edit.setUri(null);
+        if (this._toolbar)
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
         this._toolbar = new Preview.PreviewToolbar(this._preview);
@@ -448,6 +463,8 @@ const Embed = new Lang.Class({
     _prepareForEdit: function() {
         if (this._preview)
             this._preview.setModel(null);
+        if (this._toolbar)
+            this._toolbar.widget.destroy();
 
         // pack the toolbar
         this._toolbar = new Edit.EditToolbar(this._preview);
