@@ -119,6 +119,7 @@ const MainWindow = new Lang.Class({
 
         this._configureId = Mainloop.timeout_add(_CONFIGURE_ID_TIMEOUT, Lang.bind(this,
             function() {
+                this._configureId = 0;
                 this._saveWindowGeometry();
                 return false;
             }));
@@ -205,8 +206,8 @@ const MainWindow = new Lang.Class({
         let state = event.get_state()[1];
 
         let isBack = (((state & Gdk.ModifierType.MOD1_MASK) != 0 &&
-                       (direction == Gtk.TextDirection.LTR && keyval == Gdk.KEY_Left) ||
-                       (direction == Gtk.TextDirection.RTL && keyval == Gdk.KEY_Right)) ||
+                       ((direction == Gtk.TextDirection.LTR && keyval == Gdk.KEY_Left) ||
+                       (direction == Gtk.TextDirection.RTL && keyval == Gdk.KEY_Right))) ||
                       keyval == Gdk.KEY_Back);
 
         return isBack;
@@ -276,7 +277,7 @@ const MainWindow = new Lang.Class({
         aboutDialog.license_type = Gtk.License.GPL_2_0;
         aboutDialog.logo_icon_name = 'gnome-documents';
         aboutDialog.version = Config.PACKAGE_VERSION;
-        aboutDialog.website = 'http://live.gnome.org/GnomeDocuments';
+        aboutDialog.website = 'https://wiki.gnome.org/Apps/Documents';
         aboutDialog.wrap_license = true;
 
         aboutDialog.modal = true;
