@@ -67,12 +67,15 @@ const Searchbar = new Lang.Class({
                 if (keyval == Gdk.KEY_Escape) {
                     Application.application.change_action_state('search', GLib.Variant.new('b', false));
                     return true;
+                } else if (keyval == Gdk.KEY_Return) {
+                    this.emit('activate-result');
+                    return true;
                 }
 
                 return false;
             }));
 
-        this._searchEntry.connect('changed', Lang.bind(this,
+        this._searchEntry.connect('search-changed', Lang.bind(this,
             function() {
                 if (this.searchChangeBlocked)
                     return;
@@ -201,6 +204,7 @@ const Searchbar = new Lang.Class({
         this._searchEntry.set_text('');
     }
 });
+Signals.addSignalMethods(Searchbar.prototype);
 
 const Dropdown = new Lang.Class({
     Name: 'Dropdown',
@@ -285,6 +289,12 @@ const OverviewSearchbar = new Lang.Class({
         this._searchEntry = new Gd.TaggedEntry({ width_request: 500 });
         this._searchEntry.connect('tag-clicked',
             Lang.bind(this, this._onTagClicked));
+        this._searchEntry.connect('tag-button-clicked',
+            Lang.bind(this, this._onTagButtonClicked));
+
+        this._sourceTag = new Gd.TaggedEntryTag();
+        this._typeTag = new Gd.TaggedEntryTag();
+        this._matchTag = new Gd.TaggedEntryTag();
 
         // connect to search string changes in the controller
         this._searchChangedId = Application.searchController.connect('search-string-changed',
@@ -321,7 +331,7 @@ const OverviewSearchbar = new Lang.Class({
     },
 
     entryChanged: function() {
-        let currentText = this._searchEntry.get_text().toLowerCase();
+        let currentText = this._searchEntry.get_text();
 
         Application.searchController.disconnect(this._searchChangedId);
         Application.searchController.setString(currentText);
@@ -345,35 +355,43 @@ const OverviewSearchbar = new Lang.Class({
         }
     },
 
-    _onActiveChangedCommon: function(id, manager) {
+    _onActiveChangedCommon: function(id, manager, tag) {
         let item = manager.getActiveItem();
 
         if (item.id == 'all') {
-            this._searchEntry.remove_tag(id);
+            this._searchEntry.remove_tag(tag);
         } else {
-            let res = this._searchEntry.add_tag(id, item.name);
-
-            if (res) {
-                this._searchEntry.connect('tag-button-clicked::' + id, Lang.bind(this,
-                    function() {
-                        manager.setActiveItemById('all');
-                    }));
-            } else {
-                this._searchEntry.set_tag_label(id, item.name);
-            }
+            tag.set_label(item.name);
+            this._searchEntry.add_tag(tag);
         }
     },
 
     _onActiveSourceChanged: function() {
-        this._onActiveChangedCommon('source', Application.sourceManager);
+        this._onActiveChangedCommon('source', Application.sourceManager, this._sourceTag);
     },
 
     _onActiveTypeChanged: function() {
-        this._onActiveChangedCommon('type', Application.searchTypeManager);
+        this._onActiveChangedCommon('type', Application.searchTypeManager, this._typeTag);
     },
 
     _onActiveMatchChanged: function() {
-        this._onActiveChangedCommon('match', Application.searchMatchManager);
+        this._onActiveChangedCommon('match', Application.searchMatchManager, this._matchTag);
+    },
+
+    _onTagButtonClicked: function(entry, tag) {
+        let manager = null;
+
+        if (tag == this._matchTag) {
+            manager = Application.searchMatchManager;
+        } else if (tag == this._typeTag) {
+            manager = Application.searchTypeManager;
+        } else if (tag == this._sourceTag) {
+            manager = Application.sourceManager;
+        }
+
+        if (manager) {
+            manager.setActiveItemById('all');
+        }
     },
 
     _onTagClicked: function() {
