@@ -99,13 +99,6 @@ function getURNFromPath(path, model) {
     return id;
 }
 
-function alphaGtkWidget(widget) {
-    widget.override_background_color(0, new Gdk.RGBA({ red: 0,
-                                                       green: 0,
-                                                       blue: 0,
-                                                       alpha: 0 }));
-}
-
 function debug(str) {
     if (!debugInit) {
         let env = GLib.getenv('DOCUMENTS_DEBUG');
